import requests
import json
import pandas as pd
from pandas.io.json import json_normalize

from .env import (ELASTIC_DOMAIN_URL, AWS_REGION,
    DISCOVER_API_URL)
from .helpers import make_discover_request
from . import constants as cnst
from .constants import (BULK_FILES_INDEX, BULK_SAMPLE_REPO_INDEX,
                        BULK_DATA_INDEX, SC_FILES_INDEX,
                        SC_SAMPLE_REPO_INDEX, SC_DATA_INDEX)
from .download_helper import download_file


class Discover:

    def __init__(self, host = ELASTIC_DOMAIN_URL, region=AWS_REGION):
        """
        returns a discover object
        """
        self.host = host
        self.service = 'es'
        self.region = region


    def get_repositories(self):
        url = DISCOVER_API_URL+cnst.REPOSITORIES_ENDPOINT
        resp = make_discover_request('GET', url)
        if resp.status_code == cnst.OK:
            data = resp.json().get('data')
            if data:
                repositories = [each.get('attributes') for each in data]
                return repositories
        else:
            print(resp.text)

    def set_repo(self, repo_id, mode = 'bulk'):

        if not isinstance(repo_id, str):
            raise Exception('repo_id must be a str')

        if mode not in ["bulk", "single_cell"]:
            raise Exception("Mode has to be one of 'bulk' or 'single_cell'")
        
        repositories = self.get_repositories()
        repo = None
        for each in repositories:
            if each['repo_id'] == repo_id:
                repo = each
                break
        
        if repo and repo.get('indexes'):
            indexes = repo.get('indexes')
            if mode == "bulk":
                files_endpoint = indexes.get(BULK_FILES_INDEX)
                sample_repo_endpoint = indexes.get(BULK_SAMPLE_REPO_INDEX)
                data_endpoint = indexes.get(BULK_DATA_INDEX)

            if mode == "single_cell":
                files_endpoint = indexes.get(SC_FILES_INDEX)
                sample_repo_endpoint = indexes.get(SC_SAMPLE_REPO_INDEX)
                data_endpoint = indexes.get(SC_DATA_INDEX)

            self.set_sample_repo(sample_repo_endpoint)
            self.set_dataset_repo(files_endpoint)
            self.set_feature_repo(data_endpoint)
        else:
            print('Error while setting the repo. Try Discover().get_repositories()')


    def set_sample_repo(self, endpoint):
        from .sample_repo import SampleRepo
        self.sample_repo = SampleRepo(endpoint)
        self.sample_repo.host = self.host
        self.sample_repo.service = self.service
        self.sample_repo.region = self.region
        
    def set_feature_repo(self, endpoint):
        from .feature_repo import FeatureRepo
        self.feature_repo = FeatureRepo(endpoint)
        self.feature_repo.host = self.host
        self.feature_repo.service = self.service
        self.feature_repo.region = self.region

    def set_dataset_repo(self, endpoint):
        from .dataset_repo import DatasetRepo
        self.dataset_repo = DatasetRepo(endpoint)
        self.dataset_repo.host = self.host
        self.dataset_repo.service = self.service
        self.dataset_repo.region = self.region
        # hotfix 
        endpoint = endpoint.split("_")[0]+"_csv"
        from .annotation_repo import AnnotationRepo
        self.annotation_repo = AnnotationRepo(endpoint)
        self.annotation_repo.host = self.host
        self.annotation_repo.service = self.service
        self.annotation_repo.region = self.region

    def set_annotation_repo(self, endpoint):
        from .annotation_repo import AnnotationRepo
        self.annotation_repo = AnnotationRepo(endpoint)
        self.annotation_repo.host = self.host
        self.annotation_repo.service = self.service
        self.annotation_repo.region = self.region


    def __repr__(self):
        s = "Discover object"
        s = s + "\n" + f'host: {self.host}'
        s = s + "\n" + f'region: {self.region}'
        if hasattr(self, "dataset_repo"):
            s = s + "\n" + f'dataset_repo: {self.dataset_repo.endpoint}'

        if hasattr(self, "sample_repo"):
            s = s+ "\n" + f'sample_repo: {self.sample_repo.endpoint}'

        if hasattr(self, "feature_repo"):
            s = s + "\n"+ f'feature_repo: {self.feature_repo.endpoint}'

        if hasattr(self, "annotation_repo"):
            s = s + "\n" + f'annotation_repo: {self.annotation_repo.endpoint}'
        
        return s

    def get_all_fields(self):
        """
        get all fields available for querying the endpoint
        """
        url = self.host + '/' + self.endpoint
        print(url)
        r = make_discover_request('GET', url)
        output = r.json()
        fields = list(output[self.endpoint]["mappings"]["properties"].keys())
        return fields

    def get_top_n_examples(self, n = 10):
        """
        """
        url =  self.host + '/' + self.endpoint + '/_search'
        payload = {'size':n}
        r = make_discover_request('GET', url, params = payload)
        output = r.json()
        records = [each["_source"] for each in list(output["hits"]["hits"])]
        top_n_df = json_normalize(records)
        return top_n_df

    def get_file(self, key, repo_id, local_fp):
        download_file(key, repo_id, local_fp)


